function res = innerprod(X,Y)
%INNERPROD Efficient inner product with a ttensor.
%
%   R = INNERPROD(X,Y) efficiently computes the inner product between
%   two tensors X and Y. This inner product is the standard inner product,
%   if the tensors were treated as vectors. How to do this most efficiently 
%   depends on the tensor Y.
%
%   See also TENSOR/INNERPROD, TTENSOR, KTENSOR/INNERPROD
%
%MATLAB Tensor Toolbox.
%Copyright 2015, Sandia Corporation.

% This is the MATLAB Tensor Toolbox by T. Kolda, B. Bader, and others.
% http://www.sandia.gov/~tgkolda/TensorToolbox.
% Copyright (2015) Sandia Corporation. Under the terms of Contract
% DE-AC04-94AL85000, there is a non-exclusive license for use of this
% work by or on behalf of the U.S. Government. Export of this data may
% require a license from the United States Government.
% The full license terms can be found in the file LICENSE.txt

% X is a ttensor
switch class(Y)
    
    case {'ttensor'}
        if ~isequal(size(X),size(Y))
            error('X and Y must be the same size.');
        end
        if prod(size(X.core)) > prod(size(Y.core))
            % Reverse argument and call this function again so that the
            % tensor with the smaller core is the first argument. 
            res = innerprod(Y,X);
            return
        end
        W = cell(ndims(X),1);
        for n = 1:ndims(X)
            W{n} = X.u{n}'*Y.u{n};
        end
        J = ttm(Y.core, W);
        res = innerprod(X.core,J);
        return

    case {'tensor','sptensor'}
        if ~isequal(size(X),size(Y))
            error('X and Y must be the same size.');
        end
        if (prod(size(X)) < prod(size(X.core)))
            Z = full(X);
            res = innerprod(Z,Y);
            return;
        end
        Z = ttm(Y,X.u,'t');
        res = innerprod(Z, X.core);
        return

    case {'ktensor'}
        % Reverse arguments to call ktensor implementation
        res = innerprod(Y,X);
        return
        
    otherwise
        disp(['Inner product not available for class ' class(Y)]);
        return
end


